;===========================================================================================
; fullmode.asm  GoAsm assembler
; Initialisation DirectX Direct Draw en mode plein cran               
; Programm par AsmGges France                     http://www.chez.com/asmgges/index.htm
;===========================================================================================
; compile.bat:
; ..\..\bin\GoRC /r, res\fullmode.rc
; ..\..\bin\GoAsm fullmode.asm
; ..\..\bin\GoLink fullmode.obj res\fullmode.res user32.dll kernel32.dll gdi32.dll 
;                                                           winmm.dll    ddraw.dll
;===========================================================================================

  #include "ggesdx\win32a.inc"
  #include "ggesdx\gddraw.inc"

  ID_FRANCE = 17

  WSCREEN = 640
  HSCREEN = 480

;===========================================================================================
.code

start:
    invoke GetModuleHandle,NULL
    mov [wc.hInstance],eax

    invoke LoadIcon,[wc.hInstance],ID_FRANCE
    mov [wc.hIcon],eax

    invoke LoadCursor,NULL,IDC_ARROW
    mov [wc.hCursor],eax

    invoke RegisterClass,addr wc

    invoke GetSystemMetrics,SM_CXSCREEN
    mov [rectuser.right],eax           ;sauve le clipping actuel en x dans rectuser (WinMode)
    sub eax,WSCREEN
    shr eax,1
    mov [rect2.left],eax

    invoke GetSystemMetrics,SM_CYSCREEN
    mov [rectuser.bottom],eax          ;sauve le clipping actuel en y dans rectuser (WinMode)
    sub eax,HSCREEN
    shr eax,1
    mov [rect2.top],eax

    invoke CreateWindowEx,0,addr ClassName,addr WinCaption,WS_POPUP+WS_VISIBLE,\
                          0,0,0,0,0,0,[wc.hInstance],0
    mov [hWin],eax

    invoke ShowWindow,[hWin],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWin]

    ;-----------------------------
    ; Initialisation de DirectDraw
    ;-----------------------------
    invoke DirectDrawCreate,0,addr DDraw,0
    or eax,eax
    jnz >DdrawError

    ;--------------------
    ; Demande plein cran
    ;--------------------
    dd_setcooperativelevel([DDraw],[hWin],DDSCL_EXCLUSIVE+DDSCL_FULLSCREEN)
    or  eax,eax
    jnz >DdrawError

    ;---------------------------------
    ; Passage en 640x480 pixels 16bits
    ;---------------------------------
    dd_setdisplaymode([DDraw],WSCREEN,HSCREEN,16)
    or eax,eax
    jnz >DdrawError
 
    ;----------------------------------------------------
    ; Cration d'une surface primaire avec un back-buffer
    ;----------------------------------------------------
    invoke ZeroMemory,addr ddsd,sizeof DDSURFACEDESC
    mov D[ddsd.dwSize],sizeof DDSURFACEDESC
    mov D[ddsd.dwFlags],DDSD_CAPS + DDSD_BACKBUFFERCOUNT
    mov D[ddsd.ddsCaps.dwCaps],DDSCAPS_PRIMARYSURFACE+DDSCAPS_FLIP+DDSCAPS_COMPLEX
    mov D[ddsd.dwBackBufferCount],1
    dd_createsurface([DDraw],addr ddsd,addr Primary,0)
    or eax,eax
    jnz >DdrawError

    invoke ZeroMemory,addr ddscaps,sizeof DDSCAPS
    mov D[ddscaps.dwCaps],DDSCAPS_BACKBUFFER
    dds_getattachedsurface([Primary],addr ddscaps,addr Back)
    or eax,eax
    jnz >DdrawError

    invoke timeBeginPeriod,1
    mov D[FpsMax],60

    jmp >MenuLoop

Debut:


;===========================================================================================

GameLoop:

    invoke FillSurface,[Back],0000ffh

    ;----------------------------
    ; Ecrire dans la surface Back
    ;----------------------------
    dds_getdc([Back],addr hdc)
    or eax,eax
    jnz >

        invoke SetBkColor,[hdc],0ff0000h
        invoke SetTextColor,[hdc],0ffh

        mov D[rect1.left],0
        mov D[rect1.top],170
        mov D[rect1.right],WSCREEN
        mov D[rect1.bottom],230
        invoke DrawText,[hdc],addr MsgFullMode,0-1,addr rect1,\
                            DT_CENTER+DT_SINGLELINE+DT_BOTTOM

        dds_releasedc([Back],[hdc])
    :

    jmp >MessageLoop

;===========================================================================================

MenuLoop:



;===========================================================================================

MessageLoop:

    invoke PeekMessage,addr msg,0,0,0,PM_NOREMOVE
    or eax,eax
    jz  >
        invoke GetMessage,addr msg,0,0,0
        or eax,eax
        jz >ExitGame
        invoke TranslateMessage,addr msg
        invoke DispatchMessage,addr msg
        jmp <MessageLoop
    :
    cmp B[bActive],0
    jne >
        invoke WaitMessage
        jmp <MessageLoop
    :
        dds_islost([Primary])
        or eax,eax
        je  >.L1
        cmp eax,DDERR_SURFACELOST
        jne >
            dds_restore([Primary])
            jmp >.L1
        :
            jmp >ExitGame
        
    .L1:

    ;-------------------------------------------------
    ; Rgule la vitesse de rafraichissement de l'cran
    ;-------------------------------------------------
    invoke FpsLimiter,[FpsMax]

    ;-----
    ; Flip
    ;-----
    dds_flip([Primary],0,DDFLIP_WAIT)

    ;-----------------------
    ; Ecran de dessin et jeu
    ;-----------------------
    jmp GameLoop

;===========================================================================================

DdrawError:
    mov eax,addr MsgDdError
    jmp >Error

DataError:
    mov eax,addr MsgDataError

Error:
    invoke MessageBox,[hWin],eax,addr WinCaption,0
    call LibereSurf
    invoke DestroyWindow,[hWin]
    invoke PostQuitMessage,1
    jmp <MessageLoop

ExitGame:
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
;===========================================================================================

WindowProc frame @hWnd, @uMsg, @wParam, @lParam
    uses ebx,esi,edi

    cmp D[@uMsg],WM_CLOSE
    jne >.wmactivate
        call LibereSurf
        jmp >>.exit

    .wmactivate:
    cmp D[@uMsg],WM_ACTIVATE
    jne >.wmkeydown
        mov eax,[@wParam]
        mov B[bActive],al
        jmp >>.exit

    .wmkeydown:
    cmp D[@uMsg],WM_KEYDOWN
    jne >.defwindowproc
        cmp D[@wParam],VK_ESCAPE
        jne >>.exit
            ; Demande la fin de l'application
            ; -------------------------------
            invoke SendMessage,[@hWnd],WM_CLOSE,0,0
            jmp >.exit 

    .defwindowproc:
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .exit:

    mov eax,0
    ret
endf

;===========================================================================================
; Libre les surfaces
;===========================================================================================
LibereSurf:

    invoke timeEndPeriod,1

    cmp D[DDraw],0
    je  >.exit
        cmp D[Back],0
        je  >
            dds_release([Back])
            mov D[Back],0
        :
        cmp D[Primary],0
        je  >
            dds_release([Primary])
            mov D[Primary],0
        :
        dds_release([DDraw])
        mov D[DDraw],0

    .exit:
    invoke PostQuitMessage,0

    ret
;===========================================================================================

  #include "ggesdx\directxproc.asm"
  #include "ggesdx\generalproc.asm"

;===========================================================================================
.data

 ClassName     db 'AsmGges',0
 WinCaption    db ' AsmGges Win32Asm',0
 MsgDdError    db 'Direct Draw: failed',0
 MsgFullMode   db 'GoAsm DirectX Full screen mode  Exit=ESC',0
 MsgDataError  db 'Opening data file: failed',0
 align 4

 hdc           dd 0
 hWin          dd 0
 bActive       db 0
 align 4

 FpsMax        dd 0
 NextTimeEdx   dd 0
 NextTimeEax   dd 0
 PerfFreq      dq 0
 PerfCounter   dq 0

 DDraw         dd 0 

 Primary       dd 0   
 Back          dd 0   

 wc            WNDCLASS      < 0,addr WindowProc,0,0,0,0,0,0,0,addr ClassName >
 msg           MSG           <>

 ddsd          DDSURFACEDESC <>
 ddscaps       DDSCAPS       <>
 ddbltfx       DDBLTFX       <>
 ddpixelformat DDPIXELFORMAT <>

 rect1         RECT <>
 rect2         RECT <>
 rectuser      RECT <>

;===========================================================================================
;///////////////////////////////////////////////////////////////////////////////////////////
;===========================================================================================